﻿;/////////////////////////////////////////////////////////////////////////////////
;***GDI+ wrapper file***
;=======================
;
;FontFamily class.
;/////////////////////////////////////////////////////////////////////////////////

;/////////////////////////////////////////////////////////////////////////////////
;NOTE that some methods return values in 'font design units'. These units have meaning only for the font design itself and cannot
;be used when referring to any kind of physical units. You cannot even compare these values between two different font families because they may
;well refer to different resolutions.
;The key if wishing to convert to device units in some situation is in knowing that the design units remain proportional to any
;physical font. E.g. a physical font's height property will be directly proportional to it's associated font family's line-spacing property
;(reported in design units). Use this to convert other quantities (such as cell ascent) to device units when appropriate.
;
;NOTE also that GDI+ only deals with open-type fonts.
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;-INTERFACE.

  ;All methods return a gdiPlus status code (0 for okay) unless otherwise stated.
  Interface gdiPlus_FontFamily Extends gdiPlus_BaseClass
    Destroy()
    Clone.i()                                       ;Returns, if successful, a new gdiPlus_FontFamily object containing an exact copy of the original object.
    GetCellAscent.i(fontStyle=#FontStyleRegular)    ;Returns the cell ascent when the font family is given the specified style.
                                                    ;fontStye = FontStyle constant, e.g. #FontStyleRegular.
                                                    ;This is returned in font design units (see note above).
    GetCellDescent.i(fontStyle=#FontStyleRegular)   ;Returns the cell descent when the font family is given the specified style.
                                                    ;fontStye = FontStyle constant, e.g. #FontStyleRegular.
                                                    ;This is returned in font design units (see note above).
    GetEmHeight.i(fontStyle=#FontStyleRegular)      ;Returns the EM height of the font family when given the specified style.
                                                    ;fontStye = FontStyle constant, e.g. #FontStyleRegular.
                                                    ;This is returned in font design units (see note above).
    GetFamilyName.s(langID=#LANG_NEUTRAL)           ;Returns the family name.
    GetLineSpacing.i(fontStyle=#FontStyleRegular)   ;Returns the line spacing of the font family when given the specified style.
                                                    ;fontStye = FontStyle constant, e.g. #FontStyleRegular.
                                                    ;This is returned in font design units (see note above).
    IsStyleAvailable.i(fontStyle=#FontStyleRegular) ;fontStye = FontStyle constant, e.g. #FontStyleRegular.
                                                    ;Returns #True if the underlying font family supports the style specified.
  EndInterface
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;-CLASS TEMPLATE.

  Macro _gdiPlus_FontFamily_Template
    _gdiPlus_GenericObject_Template
  EndMacro
;/////////////////////////////////////////////////////////////////////////////////


;-CONSTRUCTORS.

;/////////////////////////////////////////////////////////////////////////////////
;Returns, if successful, a gdiPlus_FontFamily object pointer.
Procedure.i gdiPlus_CreateFontFamily(name$, *fontCollection._gdiPlus_GenericObject_Template)
  Protected *fontFamily._gdiPlus_FontFamily_Template, t1
  *fontFamily = AllocateMemory(SizeOf(_gdiPlus_FontFamily_Template))
  If *fontFamily
    *fontFamily\vTable = ?_gdiPlus_FontFamily_Template_VT
    If *fontCollection
      t1 = *fontCollection\object
    EndIf
    If GdipCreateFontFamilyFromName(name$, t1, @*fontFamily\object) <> #ok
      FreeMemory(*fontFamily)
      *fontFamily = 0
    EndIf
  EndIf
  ProcedureReturn *fontFamily
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;Returns, if successful, a gdiPlus_FontFamily object pointer.
Procedure.i gdiPlus_GenericMonospace()
  Protected *fontFamily._gdiPlus_FontFamily_Template
  *fontFamily = AllocateMemory(SizeOf(_gdiPlus_FontFamily_Template))
  If *fontFamily
    *fontFamily\vTable = ?_gdiPlus_FontFamily_Template_VT
    If GdipGetGenericFontFamilyMonospace(@*fontFamily\object) <> #ok
      FreeMemory(*fontFamily)
      *fontFamily = 0
    EndIf
  EndIf
  ProcedureReturn *fontFamily
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;Returns, if successful, a gdiPlus_FontFamily object pointer.
Procedure.i gdiPlus_GenericSansSerif()
  Protected *fontFamily._gdiPlus_FontFamily_Template
  *fontFamily = AllocateMemory(SizeOf(_gdiPlus_FontFamily_Template))
  If *fontFamily
    *fontFamily\vTable = ?_gdiPlus_FontFamily_Template_VT
    If GdipGetGenericFontFamilySansSerif(@*fontFamily\object) <> #ok
      FreeMemory(*fontFamily)
      *fontFamily = 0
    EndIf
  EndIf
  ProcedureReturn *fontFamily
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;Returns, if successful, a gdiPlus_FontFamily object pointer.
Procedure.i gdiPlus_GenericSerif()
  Protected *fontFamily._gdiPlus_FontFamily_Template
  *fontFamily = AllocateMemory(SizeOf(_gdiPlus_FontFamily_Template))
  If *fontFamily
    *fontFamily\vTable = ?_gdiPlus_FontFamily_Template_VT
    If GdipGetGenericFontFamilySerif(@*fontFamily\object) <> #ok
      FreeMemory(*fontFamily)
      *fontFamily = 0
    EndIf
  EndIf
  ProcedureReturn *fontFamily
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;-METHODS.

;/////////////////////////////////////////////////////////////////////////////////
Procedure gdiPlus_FontFamilyClass_Destroy(*this._gdiPlus_FontFamily_Template)
  GdipDeleteFontFamily(*this\object)
  FreeMemory(*this)
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;Returns, if successful, a gdiPlus_FontFamily object pointer.
Procedure.i gdiPlus_FontFamilyClass_Clone(*this._gdiPlus_FontFamily_Template)
  Protected *fontFamily._gdiPlus_FontFamily_Template
  *fontFamily = AllocateMemory(SizeOf(_gdiPlus_FontFamily_Template))
  If *fontFamily
    *fontFamily\vTable = ?_gdiPlus_FontFamily_Template_VT
    If GdipCloneFontFamily(*this\object, @*fontFamily\object) <> #ok
      FreeMemory(*fontFamily)
      *fontFamily = 0
    EndIf
  EndIf
  ProcedureReturn *fontFamily
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i gdiPlus_FontFamilyClass_GetCellAscent(*this._gdiPlus_FontFamily_Template, fontStyle=#FontStyleRegular)
  Protected result
  GdipGetCellAscent(*this\object, fontStyle, @result)
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i gdiPlus_FontFamilyClass_GetCellDescent(*this._gdiPlus_FontFamily_Template, fontStyle=#FontStyleRegular)
  Protected result
  GdipGetCellDescent(*this\object, fontStyle, @result)
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i gdiPlus_FontFamilyClass_GetEmHeight(*this._gdiPlus_FontFamily_Template, fontStyle=#FontStyleRegular)
  Protected result
  GdipGetEmHeight(*this\object, fontStyle, @result)
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.s gdiPlus_FontFamilyClass_GetFamilyName(*this._gdiPlus_FontFamily_Template, langID=#LANG_NEUTRAL)
  Protected result$, buffer
  buffer = AllocateMemory(#LF_FACESIZE<<1 + 2)
  If buffer
    If GdipGetFamilyName(*this\object, buffer, langID) = #ok
      result$ = PeekS(buffer, -1, #PB_Unicode)
    EndIf
    FreeMemory(buffer)
  EndIf
  ProcedureReturn result$
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i gdiPlus_FontFamilyClass_GetLineSpacing(*this._gdiPlus_FontFamily_Template, fontStyle=#FontStyleRegular)
  Protected result
  GdipGetLineSpacing(*this\object, fontStyle, @result)
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i gdiPlus_FontFamilyClass_IsStyleAvailable(*this._gdiPlus_FontFamily_Template, fontStyle=#FontStyleRegular)
  Protected result
  GdipIsStyleAvailable(*this\object, fontStyle, @result)
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;-VIRTUAL TABLE.
DataSection
  _gdiPlus_FontFamily_Template_VT:
    Data.i @gdiPlus_BaseClass_GetGdiPlusHandle()
    Data.i @gdiPlus_FontFamilyClass_Destroy()
    Data.i @gdiPlus_FontFamilyClass_Clone()
    Data.i @gdiPlus_FontFamilyClass_GetCellAscent()
    Data.i @gdiPlus_FontFamilyClass_GetCellDescent()
    Data.i @gdiPlus_FontFamilyClass_GetEmHeight()
    Data.i @gdiPlus_FontFamilyClass_GetFamilyName()
    Data.i @gdiPlus_FontFamilyClass_GetLineSpacing()
    Data.i @gdiPlus_FontFamilyClass_IsStyleAvailable()
EndDataSection
;/////////////////////////////////////////////////////////////////////////////////

; IDE Options = PureBasic 4.51 (Windows - x86)
; ExecutableFormat = Shared Dll
; CursorPosition = 66
; FirstLine = 45
; Folding = ---
; Markers = 5
; EnableUnicode
; EnableXP
; Executable = nxReportU.dll