﻿;/////////////////////////////////////////////////////////////////////////////////
;***GDI+ wrapper file***
;=======================
;
;Matrix class.
;/////////////////////////////////////////////////////////////////////////////////

;/////////////////////////////////////////////////////////////////////////////////
;NOTE a Matrix object represents a 3×3 matrix that, in turn, represents an affine transformation.
;A Matrix object stores only six of the 9 numbers in a 3×3 matrix because all 3×3 matrices that represent affine
;transformations have the same third column (0, 0, 1). A general matrix has the following form :
;  a11  a12  0
;  a21  a22  0
;   dx   dy  1
;
;where  a11  a12
;       a21  a22  is a transformation matrix
;
;and    dx
;       dy  is a translation vector.
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;-INTERFACE.

  ;All methods return a gdiPlus status code (0 for okay) unless otherwise stated.
  Interface gdiPlus_Matrix Extends gdiPlus_BaseClass
    Destroy()
    Clone.i()                                                     ;Returns, if successful, a new gdiPlus_Matrix object containing an exact copy of the original matrix.
    Equals.i(*matrix._gdiPlus_GenericObject_Template)             ;Returns #True if the elements of the specified matrix match the underlying matrix.
    GetElements.i(*elements)                                      ;*elements must point to an array of 6 floats.
    Invert.i()
    IsIdentity.i()                                                ;Returns #True if this is the identity matrix.
    IsInvertible.i()                                              ;Returns #True if the matrix is invertible.
    Multiply.i(*matrix._gdiPlus_GenericObject_Template, matrixOrder=#MatrixOrderPrepend)
                                                                  ;Prepending the multiplication means that the passed matrix transformation will be first
                                                                  ;in the order of composition.
    OffsetX.f()                                                   ;Returns the horizontal translation value of this matrix, which is the element in row 3, column 1.
    OffsetY.f()                                                   ;Returns the vertical translation value of this matrix, which is the element in row 3, column 2.
    Reset.i()                                                     ;Resets the matrix with the elements of the identity matrix.
    Rotate.i(angle.f, matrixOrder=#MatrixOrderPrepend)            ;Positive angles (degrees) for clockwise rotations.
                                                                  ;Prepending the rotation means that the rotation will be first in the order of composition.
    RotateAt.i(angle.f, *point.PointF, matrixOrder=#MatrixOrderPrepend)
                                                                  ;Updates this matrix with the product of itself and a matrix that represents rotation about a specified point.
                                                                  ;Positive angles (degrees) for clockwise rotations.
                                                                  ;Prepending the rotation means that the rotation will be first in the order of composition.    
    Scale.i(scaleX.f, scaleY.f, matrixOrder=#MatrixOrderPrepend)  ;Prepending the scaling means that the scaling will be first in the order of composition.
    SetElements.i(m11.f, m12.f, m21.f, m22.f, dx.f, dy.f)
    Shear.i(shearX.f, shearY.f, matrixOrder=#MatrixOrderPrepend)  ;Prepending the shear means that the shear will be first in the order of composition.
    TransformPointsF.i(*pts, count)                               ;*pts must point to an array of PointF structures.
    TransformPointsI.i(*pts, count)                               ;*pts must point to an array of PointI structures.
    TransformVectorsF.i(*pts, count)                              ;As for TransformPointsF() but ignoring any translation element.
    TransformVectorsI.i(*pts, count)                              ;As for TransformPointsI() but ignoring any translation element.
    Translate.i(offsetX.f, offsetY.f, matrixOrder=#MatrixOrderPrepend)
                                                                  ;Prepending the translation means that the translation will be first in the order of composition.
  EndInterface
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;-CLASS TEMPLATE.

  Macro _gdiPlus_Matrix_Template
    _gdiPlus_GenericObject_Template
  EndMacro
;/////////////////////////////////////////////////////////////////////////////////


Declare.i gdiPlus_MatrixClass_Translate(*this._gdiPlus_Matrix_Template, offsetX.f, offsetY.f, matrixOrder=#MatrixOrderPrepend)


;-CONSTRUCTORS.

;/////////////////////////////////////////////////////////////////////////////////
;Returns, if successful, a gdiPlus_Matrix object pointer.
Procedure.i gdiPlus_CreateIdentityMatrix()
  Protected *matrix._gdiPlus_Matrix_Template
  *matrix = AllocateMemory(SizeOf(_gdiPlus_Matrix_Template))
  If *matrix
    *matrix\vTable = ?_gdiPlus_Matrix_Template_VT
    If GdipCreateMatrix(@*matrix\object) <> #Ok
      FreeMemory(*matrix)
      *matrix = 0
    EndIf
  EndIf
  ProcedureReturn *matrix
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;Returns, if successful, a gdiPlus_Matrix object pointer.
Procedure.i gdiPlus_CreateMatrix(m11.f, m12.f, m21.f, m22.f, dx.f, dy.f)
  Protected *matrix._gdiPlus_Matrix_Template
  *matrix = AllocateMemory(SizeOf(_gdiPlus_Matrix_Template))
  If *matrix
    *matrix\vTable = ?_gdiPlus_Matrix_Template_VT
    If GdipCreateMatrix2(m11, m12, m21, m22, dx, dy, @*matrix\object) <> #Ok
      FreeMemory(*matrix)
      *matrix = 0
    EndIf
  EndIf
  ProcedureReturn *matrix
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;Returns, if successful, a gdiPlus_Matrix object pointer.
Procedure.i gdiPlus_CreateMatrixFromRectF(*rect.RectF, *dstplg.PointF)
  Protected *matrix._gdiPlus_Matrix_Template
  *matrix = AllocateMemory(SizeOf(_gdiPlus_Matrix_Template))
  If *matrix
    *matrix\vTable = ?_gdiPlus_Matrix_Template_VT
    If GdipCreateMatrix3(*rect, *dstplg, @*matrix\object) <> #Ok
      FreeMemory(*matrix)
      *matrix = 0
    EndIf
  EndIf
  ProcedureReturn *matrix
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;Returns, if successful, a gdiPlus_Matrix object pointer.
Procedure.i gdiPlus_CreateMatrixFromRectI(*rect.RectI, *dstplg.PointI)
  Protected *matrix._gdiPlus_Matrix_Template
  *matrix = AllocateMemory(SizeOf(_gdiPlus_Matrix_Template))
  If *matrix
    *matrix\vTable = ?_gdiPlus_Matrix_Template_VT
    If GdipCreateMatrix3I(*rect, *dstplg, @*matrix\object) <> #Ok
      FreeMemory(*matrix)
      *matrix = 0
    EndIf
  EndIf
  ProcedureReturn *matrix
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;-METHODS.

;/////////////////////////////////////////////////////////////////////////////////
Procedure gdiPlus_MatrixClass_Destroy(*this._gdiPlus_Matrix_Template)
  GdipDeleteMatrix(*this\object)
  FreeMemory(*this)
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i gdiPlus_MatrixClass_Clone(*this._gdiPlus_Matrix_Template)
  Protected *matrix._gdiPlus_Matrix_Template, t1
  If GdipCloneMatrix(*this\object, @t1) = #ok
    *matrix = AllocateMemory(SizeOf(_gdiPlus_Matrix_Template))
    If *matrix
      *matrix\vTable = ?_gdiPlus_Matrix_Template_VT
      *matrix\object = t1
    Else
      GdipDeleteMatrix(t1)
    EndIf
  EndIf
  ProcedureReturn *matrix
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i gdiPlus_MatrixClass_Equals(*this._gdiPlus_Matrix_Template, *matrix._gdiPlus_Matrix_Template)
  Protected result
  If *matrix
    GdipIsMatrixEqual(*this\object, *matrix\object, @result)
  EndIf
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i gdiPlus_MatrixClass_GetElements(*this._gdiPlus_Matrix_Template, *elements)
  Protected result
  result = GdipGetMatrixElements(*this\object, *elements)
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i gdiPlus_MatrixClass_Invert(*this._gdiPlus_Matrix_Template, *elements)
  Protected result
  result = GdipInvertMatrix(*this\object)
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i gdiPlus_MatrixClass_IsIdentity(*this._gdiPlus_Matrix_Template)
  Protected result
  GdipIsMatrixIdentity(*this\object, @result)
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i gdiPlus_MatrixClass_IsInvertible(*this._gdiPlus_Matrix_Template)
  Protected result
  GdipIsMatrixInvertible(*this\object, @result)
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i gdiPlus_MatrixClass_Multiply(*this._gdiPlus_Matrix_Template, *matrix._gdiPlus_Matrix_Template, matrixOrder=#MatrixOrderPrepend)
  Protected result
  If *matrix
    result = GdipMultiplyMatrix(*this\object, *matrix\object, matrixOrder)
  EndIf
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.f gdiPlus_MatrixClass_OffsetX(*this._gdiPlus_Matrix_Template)
  Protected result.f, Dim elements.f(5)
  If GdipGetMatrixElements(*this\object, elements()) = #ok
    result = elements(4)
  EndIf
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.f gdiPlus_MatrixClass_OffsetY(*this._gdiPlus_Matrix_Template)
  Protected result.f, Dim elements.f(5)
  If GdipGetMatrixElements(*this\object, elements()) = #ok
    result = elements(5)
  EndIf
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i gdiPlus_MatrixClass_Reset(*this._gdiPlus_Matrix_Template)
  Protected result
  result = GdipSetMatrixElements(*this\object, 1, 0, 0, 1, 0, 0)
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i gdiPlus_MatrixClass_Rotate(*this._gdiPlus_Matrix_Template, angle.f, matrixOrder=#MatrixOrderPrepend)
  Protected result
  result = GdipRotateMatrix(*this\object, angle, matrixOrder)
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i gdiPlus_MatrixClass_RotateAt(*this._gdiPlus_Matrix_Template, angle.f, *point.PointF, matrixOrder=#MatrixOrderPrepend)
  Protected result
  If *point
    Select matrixOrder 
      Case #MatrixOrderPrepend
        result = GdipTranslateMatrix(*this\object, *point\x, *point\y, #MatrixOrderPrepend)
        If result = #ok
          result = GdipRotateMatrix(*this\object, angle, #MatrixOrderPrepend)
          If result = #ok
            result = GdipTranslateMatrix(*this\object, -*point\x, -*point\y, #MatrixOrderPrepend)
          EndIf
        EndIf
      Case #MatrixOrderAppend
        result = GdipTranslateMatrix(*this\object, -*point\x, -*point\y, #MatrixOrderAppend)
        If result = #ok
          result = GdipRotateMatrix(*this\object, angle, #MatrixOrderAppend)
          If result = #ok
            result = GdipTranslateMatrix(*this\object, *point\x, *point\y, #MatrixOrderAppend)
          EndIf
        EndIf
      Default
        result = #InvalidParameter
    EndSelect
  Else
    result = #InvalidParameter
  EndIf
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i gdiPlus_MatrixClass_Scale(*this._gdiPlus_Matrix_Template, scaleX.f, scaleY.f, matrixOrder=#MatrixOrderPrepend)
  Protected result
  result = GdipScaleMatrix(*this\object, scaleX, scaleY, matrixOrder)
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i gdiPlus_MatrixClass_SetElements(*this._gdiPlus_Matrix_Template, m11.f, m12.f, m21.f, m22.f, dx.f, dy.f)
  Protected result
  result = GdipSetMatrixElements(*this\object, m11, m12, m21, m22, dx, dy)
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i gdiPlus_MatrixClass_Shear(*this._gdiPlus_Matrix_Template, shearX.f, shearY.f, matrixOrder=#MatrixOrderPrepend)
  Protected result
  result = GdipShearMatrix(*this\object, shearX, shearY, matrixOrder)
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i gdiPlus_MatrixClass_TransformPointsF(*this._gdiPlus_Matrix_Template, *pts, count)
  Protected result
  result = GdipTransformMatrixPoints(*this\object, *pts, count)
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i gdiPlus_MatrixClass_TransformPointsI(*this._gdiPlus_Matrix_Template, *pts, count)
  Protected result
  result = GdipTransformMatrixPointsI(*this\object, *pts, count)
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i gdiPlus_MatrixClass_TransformVectorsF(*this._gdiPlus_Matrix_Template, *pts, count)
  Protected result
  result = GdipVectorTransformMatrixPoints(*this\object, *pts, count)
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i gdiPlus_MatrixClass_TransformVectorsI(*this._gdiPlus_Matrix_Template, *pts, count)
  Protected result
  result = GdipVectorTransformMatrixPointsI(*this\object, *pts, count)
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
Procedure.i gdiPlus_MatrixClass_Translate(*this._gdiPlus_Matrix_Template, offsetX.f, offsetY.f, matrixOrder=#MatrixOrderPrepend)
  Protected result
  result = GdipTranslateMatrix(*this\object, offsetX.f, offsetY.f, matrixOrder)
  ProcedureReturn result
EndProcedure
;/////////////////////////////////////////////////////////////////////////////////


;/////////////////////////////////////////////////////////////////////////////////
;-VIRTUAL TABLE.
DataSection
  _gdiPlus_Matrix_Template_VT:
    Data.i @gdiPlus_BaseClass_GetGdiPlusHandle()
    Data.i @gdiPlus_MatrixClass_Destroy()
    Data.i @gdiPlus_MatrixClass_Clone()
    Data.i @gdiPlus_MatrixClass_Equals()
    Data.i @gdiPlus_MatrixClass_GetElements()
    Data.i @gdiPlus_MatrixClass_Invert()
    Data.i @gdiPlus_MatrixClass_IsIdentity()
    Data.i @gdiPlus_MatrixClass_IsInvertible()
    Data.i @gdiPlus_MatrixClass_Multiply()
    Data.i @gdiPlus_MatrixClass_OffsetX()
    Data.i @gdiPlus_MatrixClass_OffsetY()
    Data.i @gdiPlus_MatrixClass_Reset()
    Data.i @gdiPlus_MatrixClass_Rotate()
    Data.i @gdiPlus_MatrixClass_RotateAt()
    Data.i @gdiPlus_MatrixClass_Scale()
    Data.i @gdiPlus_MatrixClass_SetElements()
    Data.i @gdiPlus_MatrixClass_Shear()
    Data.i @gdiPlus_MatrixClass_TransformPointsF()
    Data.i @gdiPlus_MatrixClass_TransformPointsI()
    Data.i @gdiPlus_MatrixClass_TransformVectorsF()
    Data.i @gdiPlus_MatrixClass_TransformVectorsI()
    Data.i @gdiPlus_MatrixClass_Translate()
EndDataSection
;/////////////////////////////////////////////////////////////////////////////////

; IDE Options = PureBasic 4.51 (Windows - x86)
; ExecutableFormat = Shared Dll
; CursorPosition = 395
; FirstLine = 360
; Folding = -----
; Markers = 5
; EnableUnicode
; EnableXP
; Executable = nxReportU.dll